@@ -59,6 +59,12 @@ ffi_pl_bundle_constant(const char* package, ffi_platypus_constant_t* c)
     _uint(crypto_aead_chacha20poly1305_IETF_KEYBYTES);
     _uint(crypto_aead_chacha20poly1305_IETF_NPUBBYTES);
     _uint(crypto_aead_chacha20poly1305_IETF_ABYTES);
+
+    /* Public key Crypt - Pub Key Signatures */
+    _uint(crypto_sign_PUBLICKEYBYTES);
+    _uint(crypto_sign_SECRETKEYBYTES);
+    _uint(crypto_sign_BYTES);
+    _uint(crypto_sign_SEEDBYTES);
 }
 
 void
