@@ -2,7 +2,7 @@ package Sodium::FFI;
 use strict;
 use warnings;
 
-our $VERSION = '0.005';
+our $VERSION = '0.006';
 
 use Carp qw(croak);
 use Exporter qw(import);
@@ -33,6 +33,7 @@ push @EXPORT_OK, qw(
     crypto_aead_chacha20poly1305_ABYTES
     crypto_aead_chacha20poly1305_IETF_KEYBYTES crypto_aead_chacha20poly1305_IETF_NPUBBYTES
     crypto_aead_chacha20poly1305_IETF_ABYTES
+    crypto_sign_SEEDBYTES crypto_sign_BYTES crypto_sign_SECRETKEYBYTES crypto_sign_PUBLICKEYBYTES
 );
 
 our $ffi;
@@ -400,6 +401,155 @@ our %function = (
         }
     ],
 
+    # int
+    # crypto_sign_keypair(unsigned char *pk, unsigned char *sk);
+    'crypto_sign_keypair' => [
+        ['string', 'string'] => 'int',
+        sub {
+            my ($xsub) = @_;
+            my $pubkey = "\0" x Sodium::FFI::crypto_sign_PUBLICKEYBYTES;
+            my $seckey = "\0" x Sodium::FFI::crypto_sign_SECRETKEYBYTES;
+            my $ret = $xsub->($pubkey, $seckey);
+            if ($ret != 0) {
+                croak("Some internal error happened");
+            }
+            return ($pubkey, $seckey);
+        }
+    ],
+
+    # int
+    # crypto_sign_seed_keypair(unsigned char *pk, unsigned char *sk, const unsigned char *seed);
+    'crypto_sign_seed_keypair' => [
+        ['string', 'string', 'string'] => 'int',
+        sub {
+            my ($xsub, $seed) = @_;
+            my $seed_len = length($seed);
+            unless ($seed_len == Sodium::FFI::crypto_sign_SEEDBYTES) {
+                croak("Seed length must be crypto_sign_SEEDBYTES in length");
+            }
+            my $pubkey = "\0" x Sodium::FFI::crypto_sign_PUBLICKEYBYTES;
+            my $seckey = "\0" x Sodium::FFI::crypto_sign_SECRETKEYBYTES;
+            my $ret = $xsub->($pubkey, $seckey, $seed);
+            if ($ret != 0) {
+                croak("Some internal error happened");
+            }
+            return ($pubkey, $seckey);
+        }
+    ],
+
+    # int
+    # crypto_sign(unsigned char *sm, unsigned long long *smlen_p,
+    #     const unsigned char *m, unsigned long long mlen,
+    #     const unsigned char *sk);
+    'crypto_sign' => [
+        ['string', 'size_t*', 'string', 'size_t', 'string'] => 'int',
+        sub {
+            my ($xsub, $msg, $key) = @_;
+            my $SIZE_MAX = Sodium::FFI::SIZE_MAX;
+            my $msg_len = length($msg);
+            my $key_len = length($key);
+            unless ($key_len == Sodium::FFI::crypto_sign_SECRETKEYBYTES) {
+                croak("Secret Key length must be crypto_sign_SECRETKEYBYTES in length");
+            }
+            if ($SIZE_MAX - $msg_len <= Sodium::FFI::crypto_sign_BYTES) {
+                croak("Arithmetic overflow");
+            }
+            my $real_len = 0;
+            my $signed_len = $msg_len + Sodium::FFI::crypto_sign_BYTES;
+            my $signed = "\0" x $signed_len;
+            my $ret = $xsub->($signed, \$real_len, $msg, $msg_len, $key);
+            if ($ret != 0) {
+                croak("Some internal error happened");
+            }
+            if ($real_len >= $SIZE_MAX || $real_len > $signed_len) {
+                croak("Arithmetic overflow");
+            }
+            return substr($signed, 0, $real_len);
+        }
+    ],
+
+    # int
+    # crypto_sign_detached(unsigned char *sig, unsigned long long *siglen_p,
+    #     const unsigned char *m, unsigned long long mlen,
+    #     const unsigned char *sk);
+    'crypto_sign_detached' => [
+        ['string', 'size_t*', 'string', 'size_t', 'string'] => 'int',
+        sub {
+            my ($xsub, $msg, $key) = @_;
+            my $SIZE_MAX = Sodium::FFI::SIZE_MAX;
+            my $msg_len = length($msg);
+            my $key_len = length($key);
+            unless ($key_len == Sodium::FFI::crypto_sign_SECRETKEYBYTES) {
+                croak("Secret Key length must be crypto_sign_SECRETKEYBYTES in length");
+            }
+            my $signature = "\0" x Sodium::FFI::crypto_sign_BYTES;
+            my $real_len = 0;
+            my $ret = $xsub->($signature, \$real_len, $msg, $msg_len, $key);
+            if ($ret != 0) {
+                croak("Signature creation failed.");
+            }
+            if ($real_len <= 0 || $real_len > Sodium::FFI::crypto_sign_BYTES) {
+                croak("Signature size isn't correct.");
+            }
+            return substr($signature, 0, $real_len);
+        }
+    ],
+
+    # int
+    # crypto_sign_open(unsigned char *m, unsigned long long *mlen_p,
+    #     const unsigned char *sm, unsigned long long smlen,
+    #     const unsigned char *pk);
+    'crypto_sign_open' => [
+        ['string', 'size_t*', 'string', 'size_t', 'string'] => 'int',
+        sub {
+            my ($xsub, $msg, $key) = @_;
+            my $SIZE_MAX = Sodium::FFI::SIZE_MAX;
+            my $msg_len = length($msg);
+            my $key_len = length($key);
+            unless ($key_len == Sodium::FFI::crypto_sign_PUBLICKEYBYTES) {
+                croak("Public Key length must be crypto_sign_PUBLICKEYBYTES in length");
+            }
+            if ($SIZE_MAX - $msg_len <= Sodium::FFI::crypto_sign_BYTES) {
+                croak("Arithmetic overflow");
+            }
+            my $real_len = 0;
+            my $open = "\0" x $msg_len;
+            my $ret = $xsub->($open, \$real_len, $msg, $msg_len, $key);
+            if ($ret != 0) {
+                croak("Some internal error happened");
+            }
+            if ($real_len >= $SIZE_MAX || $real_len > $msg_len) {
+                croak("Arithmetic overflow");
+            }
+            return substr($open, 0, $real_len);
+        }
+    ],
+
+    # int
+    # crypto_sign_verify_detached(const unsigned char *sig,
+    #     const unsigned char *m,
+    #     unsigned long long mlen,
+    #     const unsigned char *pk);
+    'crypto_sign_verify_detached' => [
+        ['string', 'string', 'size_t', 'string'] => 'int',
+        sub {
+            my ($xsub, $sig, $msg, $key) = @_;
+            my $SIZE_MAX = Sodium::FFI::SIZE_MAX;
+            my $sig_len = length($sig);
+            my $msg_len = length($msg);
+            my $key_len = length($key);
+            unless ($sig_len == Sodium::FFI::crypto_sign_BYTES) {
+                croak("Signature length must be crypto_sign_BYTES in length");
+            }
+            unless ($key_len == Sodium::FFI::crypto_sign_PUBLICKEYBYTES) {
+                croak("Public Key length must be crypto_sign_PUBLICKEYBYTES in length");
+            }
+            my $ret = $xsub->($sig, $msg, $msg_len, $key);
+            return 1 if ($ret == 0);
+            return 0;
+        }
+    ],
+
     # void
     # randombytes_buf(void * const buf, const size_t size)
     'randombytes_buf' => [
@@ -1048,6 +1198,75 @@ as a string of bytes.
 The L<crypto_aead_chacha20poly1305_ietf_keygen|https://doc.libsodium.org/secret-key_cryptography/aead/chacha20-poly1305/ietf_chacha20-poly1305_construction#detached-mode>
 function returns a byte string of C<crypto_aead_chacha20poly1305_IETF_KEYBYTES> bytes.
 
+=head1 Public Key Cryptography - Public Key Signatures
+
+LibSodium provides a few
+L<Public Key Signature Functions|https://doc.libsodium.org/public-key_cryptography/public-key_signatures>
+where a signer generates a key pair (public key and secret key) and appends the secret
+key to any number of messages. The one doing the verification will need to know and trust the public key
+before messages signed using it can be verified. This is not authenticated encryption.
+
+=head2 crypto_sign
+
+    use Sodium::FFI qw(crypto_sign_keypair crypto_sign);
+    my $msg = "Let's sign this and stuff!";
+    my ($public_key, $secret_key) = crypto_sign_keypair();
+    my $signed_msg = crypto_sign($msg, $secret_key);
+
+The L<crypto_sign|https://doc.libsodium.org/public-key_cryptography/public-key_signatures#combined-mode>
+function prepends a signature to an unaltered message.
+
+=head2 crypto_sign_detached
+
+    use Sodium::FFI qw(crypto_sign_keypair crypto_sign_detached);
+    my $msg = "Let's sign this and stuff!";
+    my ($public_key, $secret_key) = crypto_sign_keypair();
+    my $signature = crypto_sign_detached($msg, $secret_key);
+
+The L<crypto_sign_detached|https://doc.libsodium.org/public-key_cryptography/public-key_signatures#detached-mode>
+function signs the message with the secret key and returns the signature.
+
+=head2 crypto_sign_keypair
+
+    use Sodium::FFI qw(crypto_sign_keypair);
+    my ($public_key, $secret_key) = crypto_sign_keypair();
+
+The L<crypto_sign_keypair|https://doc.libsodium.org/public-key_cryptography/public-key_signatures#key-pair-generation>
+function randomly generates a secret key and a corresponding public key.
+
+=head2 crypto_sign_open
+
+    use Sodium::FFI qw(crypto_sign_open);
+    # we should have the public key and signed message to open
+    my $signed_msg = ...;
+    my $public_key = ...;
+    my $msg = crypto_sign_open($signed_msg, $public_key);
+
+The L<crypto_sign_open|https://doc.libsodium.org/public-key_cryptography/public-key_signatures#combined-mode>
+function checks that a signed message has a valid signature for the public key. If so, it returns that message
+and if not, it will throw.
+
+=head2 crypto_sign_seed_keypair
+
+    use Sodium::FFI qw(crypto_sign_seed_keypair crypto_sign_SEEDBYTES randombytes_buf);
+    my $seed = randombytes_buf(crypto_sign_SEEDBYTES);
+    my ($public_key, $secret_key) = crypto_sign_seed_keypair($seed);
+
+The L<crypto_sign_seed_keypair|https://doc.libsodium.org/public-key_cryptography/public-key_signatures#key-pair-generation>
+function randomly generates a secret key deterministically derived from a single key seed and a corresponding public key.
+
+=head2 crypto_sign_verify_detached
+
+    use Sodium::FFI qw(crypto_sign_verify_detached);
+    my $signature = ...;
+    my $message = ...;
+    my $public_key = ...;
+    my $boolean = crypto_sign_verify_detached($signature, $message, $public_key);
+
+The L<crypto_sign_verify_detached|https://doc.libsodium.org/public-key_cryptography/public-key_signatures#detached-mode>
+function verifies that a signature is valid for the supplied message with public key. It returns
+a boolean value, C<1> for true, C<0> for false.
+
 =head1 Random Number Functions
 
 LibSodium provides a few
